
using System;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;
using System.Runtime.InteropServices;

public class Invaders : Form
{

    // Max coordinates of movement
    private const int maxx = 250;
    private const int maxy = 260;

    private const int invaders = 25;

    private int cur_invaders = 5;

    private int active_missiles = 0;

    Random r;

    // Invader position and direction
    private int[] flag;
    private int[] xpos;
    private int[] ypos;
    private int[] dir;

    // Missile active flag and position
    private int[] mis1f; // missile flag: 0=inactive
    private int[] mis1x;
    private int[] mis1y;

    // Player position
    private int px = 10;
    private int py = maxy-32;

    // Player missile
    private int pmf = 0; // flag: 0=inactive
    private int pmx = 0;
    private int pmy = 0;

    private int pause = 0;

    // Load invader image
    Image picInvader = Image.FromFile("invader.jpg");

    // Load player's ship image
    Image picPlayer = Image.FromFile("player.jpg");
    Thread t;

    public Invaders()
    {
        r = new Random();
        flag = new int[invaders];
        xpos = new int[invaders];
        ypos = new int[invaders];
        dir = new int[invaders];
        mis1f = new int[invaders];
        mis1x = new int[invaders];
        mis1y = new int[invaders];

        ResetGame();

        this.KeyPress += new System.Windows.Forms.KeyPressEventHandler(this.Form1_KeyPress);

        SetStyle(ControlStyles.DoubleBuffer
               | ControlStyles.AllPaintingInWmPaint
               | ControlStyles.UserPaint, true);

        t = new Thread(new ThreadStart(Run));
        t.Start();
    }

    protected void ResetGame()
    {
        int x = 60;
        int y = 10;
        int d = 0;
        for (int i = 0; i < invaders; i++)
        {
            if (i < cur_invaders)
                flag[i] = 1;
            else
                flag[i] = 0;
            xpos[i] = x;
            ypos[i] = y;
            dir[i] = 0;
            mis1f[i] = 0; // missile flag: 0=inactive
            mis1x[i] = 0;
            mis1y[i] = 0;

            if (d == 0)
            {
                x += 32;
                if (x > maxx-60)
                {
                    d = 1;
                    x = maxx-60;
                    y += 35;
                }
            }
            else
            {
                x -= 32;
                if (x < 60 )
                {
                    d = 0;
                    x = 60;
                    y += 35;
                }
            }
        }

        active_missiles = 0;

        // Player position
        px = 10;
        py = maxy - 32;
        pmf = 0; // flag: 0=inactive

        pause = 0;
    }

    [DllImport("WinMM.dll")]
    public static extern bool PlaySound(byte[] wfname, int h, int fuSound);

    //  flag values for SoundFlags argument on PlaySound
    public int SND_SYNC = 0x0000;      // play synchronously

    // (default)
    public int SND_ASYNC = 0x0001;      // play asynchronously

    public void Play(string wfname)
    {
        byte[] bname = new Byte[256];    //Max path length
        bname = System.Text.Encoding.ASCII.GetBytes(wfname);
        PlaySound(bname, 0, SND_ASYNC);
    }


    protected override void OnPaint(PaintEventArgs e)
    {
        Graphics g = e.Graphics;

        // Draw invader
        for (int i = 0; i < invaders; i++)
        {
            if (flag[i] == 1)
            {
                g.DrawImage(picInvader, xpos[i], ypos[i], 34, 25);
            }

            // Draw invader's missile (if active)
            if (mis1f[i] == 1)
            {
                Pen p = new Pen(Color.Red);
                g.DrawLine(p, mis1x[i], mis1y[i] - 5, mis1x[i], mis1y[i] + 5);
            }
        }

        // Draw player
        g.DrawImage(picPlayer, px, py, 23, 22);


        // Draw player's missile (if active)
        if (pmf == 1)
        {
            Pen p = new Pen(Color.Blue);
            g.DrawLine(p, pmx, pmy - 5, pmx, pmy + 5);
        }

        base.OnPaint(e);


        // Check for missile collision with invader
        if (pmf == 1)
        {
            for (int i = 0; i < invaders; i++)
            {
                if (flag[i]==1 &&
                    pmx >= (xpos[i] + 2) && pmx <= (xpos[i] + 23) &&
                    pmy >= (ypos[i] + 2) && pmy <= (ypos[i] + 27))
                {
                    Pen p = new Pen(Color.Orange);
                    g.DrawLine(p, xpos[i] + 2, ypos[i] + 2, xpos[i] + 23, ypos[i] + 32);
                    g.DrawLine(p, xpos[i] + 2, ypos[i] + 32, xpos[i] + 23, ypos[i] + 2);
                    Play("InvaderHit.wav");
                    flag[i] = 0;
                    pmf = 0;
                }
            }
        }

        // Check for missile collision with player
        for (int i = 0; i < invaders; i++)
        {
            // Check for invader/player crash
            if (flag[i] == 1)
            {
                if (Math.Abs(xpos[i]-px)<20 && Math.Abs(ypos[i]-py)<20)
                {
                    Pen p = new Pen(Color.Orange);
                    g.DrawLine(p, px + 2, py + 2, px + 23, py + 32);
                    g.DrawLine(p, px + 2, py + 32, px + 23, py + 2);
                    pause = 1;
                    Play("PlayerHit.wav");
                }
            }

            // Check for invader missile collision
            if (mis1f[i] == 1)
            {
                if (mis1x[i] >= (px + 2) && mis1x[i] <= (px + 22) &&
                    mis1y[i] >= (py + 2) && mis1y[i] <= (py + 27))
                {
                    Pen p = new Pen(Color.Orange);
                    g.DrawLine(p, px + 2, py + 2, px + 23, py + 32);
                    g.DrawLine(p, px + 2, py + 32, px + 23, py + 2);
                    pause = 1;
                    Play("PlayerHit.wav");
                }
            }
        }
    }

    public void Run()
    {
        while (true)
        {
            if (pause == 1) continue;

            int count = 0;
            bool end_of_row = false;
            int maindir = 0;
            for (int i = 0; i < invaders; i++)
            {
                if (flag[i] == 1)
                {
                    if (dir[i] == 0)
                    {
                        // Move invader to the right
                        if (xpos[i] + 2 > maxx)
                        {
                            end_of_row = true;
                            maindir = 1;
                        }
                    }
                    else
                    {
                        if (xpos[i] - 2 < 10)
                        {
                            end_of_row = true;
                            maindir = 0;
                        }
                    }
                }
            }

            for (int i = 0; i < invaders; i++)
            {
                if (flag[i] == 1)
                {
                    count++;
                    if (end_of_row)
                    {
                        dir[i] = maindir;
                        ypos[i] += 35;
                        if (dir[i] == 0)
                            xpos[i] += 2;
                        else
                            xpos[i] -= 2;
                    }
                    else if (dir[i] == 0)
                    {
                        // Move invader to the right
                        xpos[i] += 2;
                        if (xpos[i] > maxx)
                        {
                            ypos[i] += 35;
                            dir[i] = 1;
                        }
                    }
                    else
                    {
                        // Move invader to the left
                        xpos[i] -= 2;
                        if (xpos[i] < 10)
                        {
                            ypos[i] += 35;
                            dir[i] = 0;
                        }
                    }
                }

                // Move invader's missile
                if (mis1f[i] == 1)
                {
                    // Missile active, so move it down the screen
                    mis1y[i] += 4;
                    if (mis1y[i] > maxy)
                    {
                        mis1f[i] = 0;
                        mis1x[i] = 0;
                        mis1y[i] = 0;
                        active_missiles--;
                    }
                }
                else if (flag[i]==1)
                {
                    // Fire new missile when invader in centre position
                    mis1x[i]++;
                    int rnd = r.Next();
                    if (active_missiles<2 && mis1x[i] > 15 && (rnd&0x03)==0)
                    {
                        active_missiles++;
                        mis1f[i] = 1;
                        mis1x[i] = xpos[i] + 15;
                        mis1y[i] = ypos[i] + 25;
                    }
                }

            }

            // Move player's missile
            if (pmf == 1)
            {
                // Missile active, so move it up the screen
                pmy -= 5;
                if (pmy <= 0)
                {
                    pmf = 0;
                }
            }

            if (count == 0)
            {
                cur_invaders += 5;
                if (cur_invaders > invaders) cur_invaders = invaders;
                ResetGame();
            }


            Invalidate();
            Thread.Sleep(30);
        }
    }

    private void Form1_KeyPress(object sender, KeyPressEventArgs e)
    {
        // Player key commands: Space=fire, <=left >=right
        if (e.KeyChar.Equals(',') && px>0) px -= 10;  // Move left
        if (e.KeyChar.Equals('.') && px<maxx) px += 10; // Move right
        if (e.KeyChar.Equals(' ') && pmf==0)
        {
            // Fire missile
            pmf = 1;
            pmx = px + 15;
            pmy = py;
        }

        // Reset game if its paused using space bar
        if (e.KeyChar.Equals(' ') && pause == 1)
        {
            Thread.Sleep(700);
            ResetGame();
        }
    }

    public static void Main()
    {
        Application.Run(new Invaders());
    }
}


